import json

from .base import SQSBase


class SQSListener(SQSBase):
    def receive_messages(self, max_number_of_message=1, wait_time=5, visibility_timeout=30):
        return self.sqs_queue.receive_messages(
            MaxNumberOfMessages=max_number_of_message,
            WaitTimeSeconds=wait_time,
            VisibilityTimeout=visibility_timeout)

    def process_messages(self, max_messages=None):
        counter = 0
        for message in self.receive_messages():
            body = json.loads(message.body)
            payload = json.loads(body['Message'])
            ret = self.process_message(payload)
            if ret is True:
                message.delete()

            if max_messages is not None:
                counter += 1
                if counter >= max_messages:
                    return

    def process_message(self, message):
        self.logger.info('Processing message: {}'.format(message))



import json

from .base import SQSBase


class SQSPublisher(SQSBase):
    @staticmethod
    def translate_attributes_into_amazon_bizarre_format(attributes):
        translated_attributes = {}

        for key, value in attributes.items():
            if isinstance(value, str):
                translated_attributes[key] = {
                    'DataType': 'String',
                    'StringValue': value
                }
            elif isinstance(value, (int, float)):
                translated_attributes[key] = {
                    'DataType': 'Number',
                    'StringValue': value
                }

        return translated_attributes

    def publish(self, queue_name, payload, attributes=None):
        queue = self.get_queue(queue_name)
        attributes = self.translate_attributes_into_amazon_bizarre_format(attributes) if attributes else None

        response = queue.send_message(
            MessageAttributes=attributes,
            MessageBody=json.dumps(payload)
        )

        return response.get('MessageId')



import os
import logging

import boto3
from cached_property import cached_property


class SQSBase:
    def __init__(self, aws_access_key_id=None, aws_secret_access_key=None, aws_region=None):
        self.aws_access_key_id = aws_access_key_id or os.environ['AWS_ACCESS_KEY_ID']
        self.aws_secret_access_key = aws_secret_access_key or os.environ['AWS_SECRET_ACCESS_KEY']
        self.aws_region = aws_region or os.environ['AWS_REGION']

        self.logger = logging.getLogger()

    @cached_property
    def sqs_client(self):
        return boto3.resource(
            'sqs',
            self.aws_region,
            aws_access_key_id=self.aws_access_key_id,
            aws_secret_access_key=self.aws_secret_access_key,
        )

    def get_queue(self, name):
        return self.sqs_client.get_queue_by_name(QueueName=name)
